namespace GossamerVR.MorphFilterPlus
{
    using System;
    using System.Collections.Generic;
    using UnityEngine;
    using UnityEngine.UI;

    /// <summary>
    /// A container representing a duplication of a button to replace
    /// an existing VAM UI button or render on top to intercept a click. 
    /// Has a public click event and can optionally forward the click on to the original button. 
    /// this is used to ensure we restore morphs before performing the normal button action.
    /// </summary>
    public class ButtonBridge : UIElementBridge<Button>
    {
        
        public ButtonBridge(Button oldElement, bool hideOldElement, bool forwardEvents) : base(oldElement, hideOldElement, forwardEvents) { }


        protected override void InitNewElement()
        {
            this.NewElement.onClick.RemoveAllListeners();
            this.NewElement.onClick.AddListener(this.OnClick);
            this.NewElement.interactable = true;
        }

        protected override void SetupMimic()
        {
            ColorBlock colors = ColorBlock.defaultColorBlock;
            colors.normalColor = Color.clear;
            colors.highlightedColor = new Color(0.8f, 0.8f, 0.8f, 0.3f);
            colors.pressedColor = colors.highlightedColor;
            colors.disabledColor = new Color(0.5f, 0.5f, 0.5f, 0.3f);
            this.NewElement.colors = colors;

            // we don't want to render text, as we want the old button text to be visible.
            foreach (var label in this.NewElement.GetComponentsInChildren<Text>())
            {
                label.text = "";
            }
        }

        /// <summary>
        /// The action to perform when clicking the button. subscribed in InitNewElement()
        /// </summary>
        private void OnClick()
        {
            if (!this.isMimic || this.OldElement == null || this.OldElement.interactable)
            {
                this.Interact();
                if (this.forwardEvents && this.OldElement != null && this.OldElement.onClick != null)
                {
                    this.OldElement.onClick.Invoke();
                }
            }

        }

        protected override void OnRemove()
        {
            if (this.NewElement != null)
            {
                this.NewElement.onClick.RemoveListener(this.OnClick);
            }
        }

    }
}
