namespace GossamerVR.MorphFilterPlus
{
    using System;
    using System.Collections.Generic;
    using UnityEngine;
    using UnityEngine.UI;

    /// <summary>
    /// A container representing a duplication of a button to replace
    /// an existing VAM UI button or render on top to intercept a click. 
    /// Has a public click event and can optionally forward the click on to the original button. 
    /// this is used to ensure we restore morphs before performing the normal button action.
    /// </summary>
    public class ToggleBridge : UIElementBridge<Toggle>
    {
        private bool initialValue;
        public ToggleBridge(Toggle oldElement, bool initialValue, bool hideOldElement, bool forwardEvents) : base(oldElement, hideOldElement, forwardEvents) 
        {
            this.NewElement.isOn = initialValue;
        }
        

        protected override void InitNewElement()
        {            
            this.NewElement.onValueChanged.RemoveAllListeners();
            this.NewElement.onValueChanged.AddListener(this.OnToggle);
            this.NewElement.interactable = true;
        }

        protected override void SetupMimic()
        {
            //foreach(var image in this.NewElement.GetComponentsInChildren<Image>())
            //{
            //    image.color = Color.clear;
            //}            
            foreach (var label in this.NewElement.GetComponentsInChildren<Text>())
            {
                label.text = "        ";
            }
        }

        /// <summary>
        /// The action to perform when clicking the toggle. subscribed in InitNewElement()
        /// </summary>
        private void OnToggle(bool value)
        {            
            if (!this.isMimic || this.OldElement == null || this.OldElement.interactable)
            {
                this.Interact();
                if (this.forwardEvents && this.OldElement != null)
                {
                    this.OldElement.isOn = this.NewElement.isOn;
                }
            }

        }

        protected override void OnRemove()
        {
            if (this.NewElement != null)
            {
                this.NewElement.onValueChanged.RemoveListener(this.OnToggle);
            }
        }

    }
}
